class StatusService:
    def __init__(self, client):
        self.client = client



    def getStatus(self, headers=None, query_params=None, content_type="application/json"):
        """
        Get system version, status, and uptime. This returns the current status of the system. Unlike most of the API it is avilable without authentication, though some fields are omitted in
        this case.
        It is method for GET /status
        """
        uri = self.client.base_url + "/status"
        return self.client.get(uri, None, headers, query_params, content_type)



import requests

from .network_service import NetworkService 
from .self_service import SelfService 
from .status_service import StatusService 
from .user_service import UserService 


class Client:
    def __init__(self, base_uri="https://my.zerotier.com/api"):
        self.base_url = base_uri
        self.session = requests.Session()
        
        self.network = NetworkService(self)
        self.self = SelfService(self)
        self.status = StatusService(self)
        self.user = UserService(self)

    def is_goraml_class(self, data):
        # check if a data is go-raml generated class
        # we currently only check the existence
        # of as_json method
        op = getattr(data, "as_json", None)
        if callable(op):
            return True
        return False

    def set_auth_header(self, val):
        ''' set authorization header value'''
        self.session.headers.update({"Authorization": val})

    def _get_headers(self, headers, content_type):
        if content_type:
            contentheader = {"Content-Type": content_type}
            if headers is None:
                headers = contentheader
            else:
                headers.update(contentheader)
        return headers

    def _handle_data(self, uri, data, headers, params, content_type, method):
        headers = self._get_headers(headers, content_type)
        if self.is_goraml_class(data):
            data = data.as_json()

        if content_type == "multipart/form-data":
            # when content type is multipart/formdata remove the content-type header
            # as requests will set this itself with correct boundary
            headers.pop('Content-Type')
            res = method(uri, files=data, headers=headers, params=params)
        elif data is None:
            res = method(uri, headers=headers, params=params)
        elif type(data) is str:
            res = method(uri, data=data, headers=headers, params=params)
        else:
            res = method(uri, json=data, headers=headers, params=params)
        res.raise_for_status()
        return res

    def post(self, uri, data, headers, params, content_type):
        return self._handle_data(uri, data, headers, params, content_type, self.session.post)

    def put(self, uri, data, headers, params, content_type):
        return self._handle_data(uri, data, headers, params, content_type, self.session.put)

    def patch(self, uri, data, headers, params, content_type):
        return self._handle_data(uri, data, headers, params, content_type, self.session.patch)

    def get(self, uri, data, headers, params, content_type):
        return self._handle_data(uri, data, headers, params, content_type, self.session.get)

    def delete(self, uri, data, headers, params, content_type):
        return self._handle_data(uri, data, headers, params, content_type, self.session.delete)


"""
Auto-generated class for NetworkRule
"""

from . import client_support


class NetworkRule(object):
    """
    auto-generated. don't touch.
    """

    @staticmethod
    def create(action=None, etherType=None, ruleNo=None):
        """
        :type action: str
        :type etherType: int
        :type ruleNo: int
        :rtype: NetworkRule
        """

        return NetworkRule(
            action=action,
            etherType=etherType,
            ruleNo=ruleNo,
        )

    def __init__(self, json=None, **kwargs):
        if json is None and not kwargs:
            raise ValueError('No data or kwargs present')

        class_name = 'NetworkRule'
        create_error = '{cls}: unable to create {prop} from value: {val}: {err}'
        required_error = '{cls}: missing required property {prop}'

        data = json or kwargs

        property_name = 'action'
        val = data.get(property_name)
        if val is not None:
            datatypes = [str]
            try:
                self.action = client_support.val_factory(val, datatypes)
            except ValueError as err:
                raise ValueError(create_error.format(cls=class_name, prop=property_name, val=val, err=err))

        property_name = 'etherType'
        val = data.get(property_name)
        if val is not None:
            datatypes = [int]
            try:
                self.etherType = client_support.val_factory(val, datatypes)
            except ValueError as err:
                raise ValueError(create_error.format(cls=class_name, prop=property_name, val=val, err=err))

        property_name = 'ruleNo'
        val = data.get(property_name)
        if val is not None:
            datatypes = [int]
            try:
                self.ruleNo = client_support.val_factory(val, datatypes)
            except ValueError as err:
                raise ValueError(create_error.format(cls=class_name, prop=property_name, val=val, err=err))

    def __str__(self):
        return self.as_json(indent=4)

    def as_json(self, indent=0):
        return client_support.to_json(self, indent=indent)

    def as_dict(self):
        return client_support.to_dict(self)



"""
support methods for python clients
"""

import json
import collections
from datetime import datetime
from uuid import UUID
from enum import Enum
from dateutil import parser


# python2/3 compatible basestring, for use in to_dict
try:
    basestring
except NameError:
    basestring = str


def timestamp_from_datetime(datetime):
    """
        Convert from datetime format to timestamp format
        Input: Time in datetime format
        Output: Time in timestamp format
    """
    return datetime.strftime('%Y-%m-%dT%H:%M:%S.%fZ')


def timestamp_to_datetime(timestamp):
    """
        Convert from timestamp format to datetime format
        Input: Time in timestamp format
        Output: Time in datetime format
    """
    return parser.parse(timestamp).replace(tzinfo=None)


def has_properties(cls, property, child_properties):
    for child_prop in child_properties:
        if getattr(property, child_prop, None) is None:
            return False

    return True


def list_factory(val, member_type):
    if not isinstance(val, list):
        raise ValueError('list_factory: value must be a list')
    return [val_factory(v, member_type) for v in val]


def dict_factory(val, objmap):
    # objmap is a dict outlining the structure of this value
    # its format is {'attrname': {'datatype': [type], 'required': bool}}
    objdict = {}
    for attrname, attrdict in objmap.items():
        value = val.get(attrname)
        if value is not None:
            for dt in attrdict['datatype']:
                try:
                    if isinstance(dt, dict):
                        objdict[attrname] = dict_factory(value, attrdict)
                    else:
                        objdict[attrname] = val_factory(value, [dt])
                except Exception:
                    pass
            if objdict.get(attrname) is None:
                raise ValueError('dict_factory: {attr}: unable to instantiate with any supplied type'.format(attr=attrname))
        elif attrdict.get('required'):
            raise ValueError('dict_factory: {attr} is required'.format(attr=attrname))

    return objdict


def val_factory(val, datatypes):
    """
    return an instance of `val` that is of type `datatype`.
    keep track of exceptions so we can produce meaningful error messages.
    """
    exceptions = []
    for dt in datatypes:
        try:
            if isinstance(val, dt):
                return val
            return type_handler_object(val, dt)
        except Exception as e:
            exceptions.append(str(e))
    # if we get here, we never found a valid value. raise an error
    raise ValueError('val_factory: Unable to instantiate {val} from types {types}. Exceptions: {excs}'.
                     format(val=val, types=datatypes, excs=exceptions))


def to_json(cls, indent=0):
    """
    serialize to JSON
    :rtype: str
    """
    # for consistency, use as_dict then go to json from there
    return json.dumps(cls.as_dict(), indent=indent)


def to_dict(cls, convert_datetime=True):
    """
    return a dict representation of the Event and its sub-objects
    `convert_datetime` controls whether datetime objects are converted to strings or not
    :rtype: dict
    """
    def todict(obj):
        """
        recurse the objects and represent as a dict
        use the registered handlers if possible
        """
        data = {}
        if isinstance(obj, dict):
            for (key, val) in obj.items():
                data[key] = todict(val)
            return data
        if not convert_datetime and isinstance(obj, datetime):
            return obj
        elif type_handler_value(obj):
            return type_handler_value(obj)
        elif isinstance(obj, collections.Sequence) and not isinstance(obj, basestring):
            return [todict(v) for v in obj]
        elif hasattr(obj, "__dict__"):
            for key, value in obj.__dict__.items():
                if not callable(value) and not key.startswith('_'):
                    data[key] = todict(value)
            return data
        else:
            return obj

    return todict(cls)


class DatetimeHandler(object):
    """
    output datetime objects as iso-8601 compliant strings
    """
    @classmethod
    def flatten(cls, obj):
        """flatten"""
        return timestamp_from_datetime(obj)

    @classmethod
    def restore(cls, data):
        """restore"""
        return timestamp_to_datetime(data)


class UUIDHandler(object):
    """
    output UUID objects as a string
    """
    @classmethod
    def flatten(cls, obj):
        """flatten"""
        return str(obj)

    @classmethod
    def restore(cls, data):
        """restore"""
        return UUID(data)


class EnumHandler(object):
    """
    output Enum objects as their value
    """
    @classmethod
    def flatten(cls, obj):
        """flatten"""
        return obj.value

    @classmethod
    def restore(cls, data):
        """
        cannot restore here because we don't know what type of enum it is
        """
        raise NotImplementedError


handlers = {
    datetime: DatetimeHandler,
    Enum: EnumHandler,
    UUID: UUIDHandler,
}


def handler_for(obj):
    """return the handler for the object type"""
    for handler_type in handlers:
        if isinstance(obj, handler_type):
            return handlers[handler_type]

    try:
        for handler_type in handlers:
            if issubclass(obj, handler_type):
                return handlers[handler_type]
    except TypeError:
        # if obj isn't a class, issubclass will raise a TypeError
        pass


def type_handler_value(obj):
    """
    return the serialized (flattened) value from the registered handler for the type
    """
    handler = handler_for(obj)
    if handler:
        return handler().flatten(obj)


def type_handler_object(val, objtype):
    """
    return the deserialized (restored) value from the registered handler for the type
    """
    handler = handlers.get(objtype)
    if handler:
        return handler().restore(val)
    else:
        return objtype(val)


"""
Auto-generated class for NetworkRelay
"""

from . import client_support


class NetworkRelay(object):
    """
    auto-generated. don't touch.
    """

    @staticmethod
    def create(address=None, phyAddress=None):
        """
        :type address: str
        :type phyAddress: str
        :rtype: NetworkRelay
        """

        return NetworkRelay(
            address=address,
            phyAddress=phyAddress,
        )

    def __init__(self, json=None, **kwargs):
        if json is None and not kwargs:
            raise ValueError('No data or kwargs present')

        class_name = 'NetworkRelay'
        create_error = '{cls}: unable to create {prop} from value: {val}: {err}'
        required_error = '{cls}: missing required property {prop}'

        data = json or kwargs

        property_name = 'address'
        val = data.get(property_name)
        if val is not None:
            datatypes = [str]
            try:
                self.address = client_support.val_factory(val, datatypes)
            except ValueError as err:
                raise ValueError(create_error.format(cls=class_name, prop=property_name, val=val, err=err))

        property_name = 'phyAddress'
        val = data.get(property_name)
        if val is not None:
            datatypes = [str]
            try:
                self.phyAddress = client_support.val_factory(val, datatypes)
            except ValueError as err:
                raise ValueError(create_error.format(cls=class_name, prop=property_name, val=val, err=err))

    def __str__(self):
        return self.as_json(indent=4)

    def as_json(self, indent=0):
        return client_support.to_json(self, indent=indent)

    def as_dict(self):
        return client_support.to_dict(self)



"""
Auto-generated class for Network
"""
from .NetworkConfig import NetworkConfig

from . import client_support


class Network(object):
    """
    auto-generated. don't touch.
    """

    @staticmethod
    def create(annot=None, config=None, id=None, ui=None):
        """
        :type annot: dict
        :type config: NetworkConfig
        :type id: str
        :type ui: dict
        :rtype: Network
        """

        return Network(
            annot=annot,
            config=config,
            id=id,
            ui=ui,
        )

    def __init__(self, json=None, **kwargs):
        if json is None and not kwargs:
            raise ValueError('No data or kwargs present')

        class_name = 'Network'
        create_error = '{cls}: unable to create {prop} from value: {val}: {err}'
        required_error = '{cls}: missing required property {prop}'

        data = json or kwargs

        property_name = 'annot'
        val = data.get(property_name)
        if val is not None:
            datatypes = [dict]
            try:
                self.annot = client_support.val_factory(val, datatypes)
            except ValueError as err:
                raise ValueError(create_error.format(cls=class_name, prop=property_name, val=val, err=err))

        property_name = 'config'
        val = data.get(property_name)
        if val is not None:
            datatypes = [NetworkConfig]
            try:
                self.config = client_support.val_factory(val, datatypes)
            except ValueError as err:
                raise ValueError(create_error.format(cls=class_name, prop=property_name, val=val, err=err))

        property_name = 'id'
        val = data.get(property_name)
        if val is not None:
            datatypes = [str]
            try:
                self.id = client_support.val_factory(val, datatypes)
            except ValueError as err:
                raise ValueError(create_error.format(cls=class_name, prop=property_name, val=val, err=err))

        property_name = 'ui'
        val = data.get(property_name)
        if val is not None:
            datatypes = [dict]
            try:
                self.ui = client_support.val_factory(val, datatypes)
            except ValueError as err:
                raise ValueError(create_error.format(cls=class_name, prop=property_name, val=val, err=err))

    def __str__(self):
        return self.as_json(indent=4)

    def as_json(self, indent=0):
        return client_support.to_json(self, indent=indent)

    def as_dict(self):
        return client_support.to_dict(self)



import requests


from .EnumUserAuth import EnumUserAuth
from .Error import Error
from .Network import Network
from .NetworkConfig import NetworkConfig
from .NetworkDeleteStatus import NetworkDeleteStatus
from .NetworkIpAssignmentPool import NetworkIpAssignmentPool
from .NetworkMember import NetworkMember
from .NetworkMemberConfig import NetworkMemberConfig
from .NetworkRelay import NetworkRelay
from .NetworkRule import NetworkRule
from .SystemStatus import SystemStatus
from .User import User

from .client import Client as APIClient


class Client:
    def __init__(self, base_uri="https://my.zerotier.com/api"):
        self.api = APIClient(base_uri)
        


"""
Auto-generated class for User
"""
from .EnumUserAuth import EnumUserAuth

from . import client_support


class User(object):
    """
    auto-generated. don't touch.
    """

    @staticmethod
    def create(auth, authUserId, id, state, type, _type=None, annot=None, config=None, displayName=None, email=None, emailConfirmed=None, globalPermissionMask=None, password=None, paymentProcessorCustomerId=None, permissions=None, tokens=None, ui=None):
        """
        :type _type: str
        :type annot: dict
        :type auth: EnumUserAuth
        :type authUserId: str
        :type config: dict
        :type displayName: str
        :type email: str
        :type emailConfirmed: bool
        :type globalPermissionMask: int
        :type id: str
        :type password: str
        :type paymentProcessorCustomerId: str
        :type permissions: dict
        :type state: int
        :type tokens: dict
        :type type: int
        :type ui: dict
        :rtype: User
        """

        return User(
            _type=_type,
            annot=annot,
            auth=auth,
            authUserId=authUserId,
            config=config,
            displayName=displayName,
            email=email,
            emailConfirmed=emailConfirmed,
            globalPermissionMask=globalPermissionMask,
            id=id,
            password=password,
            paymentProcessorCustomerId=paymentProcessorCustomerId,
            permissions=permissions,
            state=state,
            tokens=tokens,
            type=type,
            ui=ui,
        )

    def __init__(self, json=None, **kwargs):
        if json is None and not kwargs:
            raise ValueError('No data or kwargs present')

        class_name = 'User'
        create_error = '{cls}: unable to create {prop} from value: {val}: {err}'
        required_error = '{cls}: missing required property {prop}'

        data = json or kwargs

        property_name = '_type'
        val = data.get(property_name)
        if val is not None:
            datatypes = [str]
            try:
                self._type = client_support.val_factory(val, datatypes)
            except ValueError as err:
                raise ValueError(create_error.format(cls=class_name, prop=property_name, val=val, err=err))

        property_name = 'annot'
        val = data.get(property_name)
        if val is not None:
            datatypes = [dict]
            try:
                self.annot = client_support.val_factory(val, datatypes)
            except ValueError as err:
                raise ValueError(create_error.format(cls=class_name, prop=property_name, val=val, err=err))

        property_name = 'auth'
        val = data.get(property_name)
        if val is not None:
            datatypes = [EnumUserAuth]
            try:
                self.auth = client_support.val_factory(val, datatypes)
            except ValueError as err:
                raise ValueError(create_error.format(cls=class_name, prop=property_name, val=val, err=err))
        else:
            raise ValueError(required_error.format(cls=class_name, prop=property_name))

        property_name = 'authUserId'
        val = data.get(property_name)
        if val is not None:
            datatypes = [str]
            try:
                self.authUserId = client_support.val_factory(val, datatypes)
            except ValueError as err:
                raise ValueError(create_error.format(cls=class_name, prop=property_name, val=val, err=err))
        else:
            raise ValueError(required_error.format(cls=class_name, prop=property_name))

        property_name = 'config'
        val = data.get(property_name)
        if val is not None:
            datatypes = [dict]
            try:
                self.config = client_support.val_factory(val, datatypes)
            except ValueError as err:
                raise ValueError(create_error.format(cls=class_name, prop=property_name, val=val, err=err))

        property_name = 'displayName'
        val = data.get(property_name)
        if val is not None:
            datatypes = [str]
            try:
                self.displayName = client_support.val_factory(val, datatypes)
            except ValueError as err:
                raise ValueError(create_error.format(cls=class_name, prop=property_name, val=val, err=err))

        property_name = 'email'
        val = data.get(property_name)
        if val is not None:
            datatypes = [str]
            try:
                self.email = client_support.val_factory(val, datatypes)
            except ValueError as err:
                raise ValueError(create_error.format(cls=class_name, prop=property_name, val=val, err=err))

        property_name = 'emailConfirmed'
        val = data.get(property_name)
        if val is not None:
            datatypes = [bool]
            try:
                self.emailConfirmed = client_support.val_factory(val, datatypes)
            except ValueError as err:
                raise ValueError(create_error.format(cls=class_name, prop=property_name, val=val, err=err))

        property_name = 'globalPermissionMask'
        val = data.get(property_name)
        if val is not None:
            datatypes = [int]
            try:
                self.globalPermissionMask = client_support.val_factory(val, datatypes)
            except ValueError as err:
                raise ValueError(create_error.format(cls=class_name, prop=property_name, val=val, err=err))

        property_name = 'id'
        val = data.get(property_name)
        if val is not None:
            datatypes = [str]
            try:
                self.id = client_support.val_factory(val, datatypes)
            except ValueError as err:
                raise ValueError(create_error.format(cls=class_name, prop=property_name, val=val, err=err))
        else:
            raise ValueError(required_error.format(cls=class_name, prop=property_name))

        property_name = 'password'
        val = data.get(property_name)
        if val is not None:
            datatypes = [str]
            try:
                self.password = client_support.val_factory(val, datatypes)
            except ValueError as err:
                raise ValueError(create_error.format(cls=class_name, prop=property_name, val=val, err=err))

        property_name = 'paymentProcessorCustomerId'
        val = data.get(property_name)
        if val is not None:
            datatypes = [str]
            try:
                self.paymentProcessorCustomerId = client_support.val_factory(val, datatypes)
            except ValueError as err:
                raise ValueError(create_error.format(cls=class_name, prop=property_name, val=val, err=err))

        property_name = 'permissions'
        val = data.get(property_name)
        if val is not None:
            datatypes = [dict]
            try:
                self.permissions = client_support.val_factory(val, datatypes)
            except ValueError as err:
                raise ValueError(create_error.format(cls=class_name, prop=property_name, val=val, err=err))

        property_name = 'state'
        val = data.get(property_name)
        if val is not None:
            datatypes = [int]
            try:
                self.state = client_support.val_factory(val, datatypes)
            except ValueError as err:
                raise ValueError(create_error.format(cls=class_name, prop=property_name, val=val, err=err))
        else:
            raise ValueError(required_error.format(cls=class_name, prop=property_name))

        property_name = 'tokens'
        val = data.get(property_name)
        if val is not None:
            datatypes = [dict]
            try:
                self.tokens = client_support.val_factory(val, datatypes)
            except ValueError as err:
                raise ValueError(create_error.format(cls=class_name, prop=property_name, val=val, err=err))

        property_name = 'type'
        val = data.get(property_name)
        if val is not None:
            datatypes = [int]
            try:
                self.type = client_support.val_factory(val, datatypes)
            except ValueError as err:
                raise ValueError(create_error.format(cls=class_name, prop=property_name, val=val, err=err))
        else:
            raise ValueError(required_error.format(cls=class_name, prop=property_name))

        property_name = 'ui'
        val = data.get(property_name)
        if val is not None:
            datatypes = [dict]
            try:
                self.ui = client_support.val_factory(val, datatypes)
            except ValueError as err:
                raise ValueError(create_error.format(cls=class_name, prop=property_name, val=val, err=err))

    def __str__(self):
        return self.as_json(indent=4)

    def as_json(self, indent=0):
        return client_support.to_json(self, indent=indent)

    def as_dict(self):
        return client_support.to_dict(self)



from setuptools import setup, find_packages
# To use a consistent encoding
from codecs import open
from os import path

here = path.abspath(path.dirname(__file__))


setup(
    name='zerotier',
    version='1.1.2',
    description='Zerotier API client',
    url='https://github.com/zero-os/zerotier_client',
    author='GIG',
    author_email='info@gig.tech',
    license='Apache 2.0',
    packages=find_packages(),
    install_requires=['requests'],
)
